/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Brutus e-book backend class.
 *
 *    Authors:
 *        Jules Colding <colding@42tools.com>
 *
 *    Copyright (C) 2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA 02111-1307 USA
 */

#ifndef __E_BOOK_BACKEND_BRUTUS_H__
#define __E_BOOK_BACKEND_BRUTUS_H__

#include <libedata-book/e-book-backend.h>
#include <libedata-book/e-book-backend-db-cache.h>
#include <libedata-book/e-book-backend-summary.h>

#include <idl_output/IMAPISession.h>
#include <idl_output/IAddrBook.h>

#include <session/brutusd-util.h>
#include <session/brutus-proxy.h>
#include <server/brutus.h>

G_BEGIN_DECLS

#define E_TYPE_BOOK_BACKEND_BRUTUS        (e_book_backend_brutus_get_type ())
#define E_BOOK_BACKEND_BRUTUS(o)          (G_TYPE_CHECK_INSTANCE_CAST ((o), E_TYPE_BOOK_BACKEND_BRUTUS, EBookBackendBrutus))
#define E_BOOK_BACKEND_BRUTUS_CLASS(k)    (G_TYPE_CHECK_CLASS_CAST((k), E_TYPE_BOOK_BACKEND_BRUTUS, EBookBackendBrutusClass))
#define E_IS_BOOK_BACKEND_BRUTUS(o)       (G_TYPE_CHECK_INSTANCE_TYPE ((o), E_TYPE_BOOK_BACKEND_BRUTUS))
#define E_IS_BOOK_BACKEND_BRUTUS_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), E_TYPE_BOOK_BACKEND_BRUTUS))
#define E_BOOK_BACKEND_BRUTUS_GET_CLASS(k) (G_TYPE_INSTANCE_GET_CLASS ((obj), E_TYPE_BOOK_BACKEND_BRUTUS, EBookBackenBrutusClass))

typedef enum {
	ADDRESS_BOOK_PAB,
	ADDRESS_BOOK_GAL
} ADDRESS_BOOK_KIND;

typedef struct _BrutusEBookSearchDef {
	size_t tag_count;
	char *search_token;
	BRUTUS_BDEFINE *search_tags;
} BrutusEBookSearchDef;

typedef struct _BrutusSearchDef {
	size_t len;
	BrutusEBookSearchDef *mapping;
} BrutusSearchDef;

typedef struct _EBookBackendBrutusPrivate {
	BrutusBaseClass base_class;  // basic data required to connect to Brutus server

	long auto_check_timeout;     // auto check timeout interval in minutes
	guint timeout_id;            // ID of timing source
	GMutex *poll_mutex;

	gboolean constructed;        // initialized with ORB, POA and all
	gboolean make_offline_cache; // create an offline copy of the addressbook
	gboolean is_writable;
	GNOME_Evolution_Addressbook_BookMode mode; // offline (GNOME_Evolution_Addressbook_MODE_LOCAL) or online (GNOME_Evolution_Addressbook_MODE_REMOTE)

	// Personal Address Book
	BRUTUS_IMsgStore msg_store;
	BRUTUS_IMAPIFolder PAB_book;

	// Global Address List
	BRUTUS_IAddrBook addrbook;
	BRUTUS_IABContainer GAL_book;

	DB     *cache;
	DB_ENV *cache_env;

	gboolean cache_ready;
	gboolean summary_ready;

	gchar *timestamp_file_name; // stringified UTC server timestamp 
	GMutex *timestamp_mutex;    // serializes access to the timestamp file

	gchar *uri;
	gchar *book_name;           // "GAL", "PAB" or a user specific name
	gchar *summary_file_name;
	EBookBackendSummary *summary;

	ADDRESS_BOOK_KIND kind;
	BRUTUS_BDEFINE *content_tags; // book specific property tags to include in content tables including PR_ENTRYID
	BrutusSearchDef *search_def;  // book specific search mapping
} EBookBackendBrutusPrivate;

typedef struct _EBookBackendBrutus {
	EBookBackend backend;

	/* Private data */
	EBookBackendBrutusPrivate *priv;
} EBookBackendBrutus;

typedef struct _EBookBackendBrutusClass {
	EBookBackendClass parent_class;
} EBookBackendBrutusClass;


extern EBookBackend *e_book_backend_brutus_new(void);

extern GType e_book_backend_brutus_get_type(void);

G_END_DECLS

#endif /* __E_BOOK_BACKEND_BRUTUS_H__ */
